@@ -1097,7 +1097,7 @@ void I830DRI2FlipEventHandler(unsigned int frame, unsigned int tv_sec,
 				i830_dri2_del_frame_event(chain_drawable, chain);
 			} else if (!can_exchange(chain_drawable, chain->front, chain->back) ||
 				   !I830DRI2ScheduleFlip(intel, chain_drawable, chain)) {
-				I830DRI2FallbackBlitSwap(drawable,
+				I830DRI2FallbackBlitSwap(chain_drawable,
 							 chain->front,
 							 chain->back);
 
@@ -1326,6 +1326,16 @@ blit_fallback:
 	return TRUE;
 }
 
+static uint64_t gettime_us(void)
+{
+	struct timespec tv;
+
+	if (clock_gettime(CLOCK_MONOTONIC, &tv))
+		return 0;
+
+	return (uint64_t)tv.tv_sec * 1000000 + tv.tv_nsec / 1000;
+}
+
 /*
  * Get current frame count and frame count timestamp, based on drawable's
  * crtc.
@@ -1339,9 +1349,9 @@ I830DRI2GetMSC(DrawablePtr draw, CARD64 *ust, CARD64 *msc)
 	drmVBlank vbl;
 	int ret, pipe = I830DRI2DrawablePipe(draw);
 
-	/* Drawable not displayed, make up a value */
+	/* Drawable not displayed, make up a *monotonic* value */
 	if (pipe == -1) {
-		*ust = 0;
+		*ust = gettime_us();
 		*msc = 0;
 		return TRUE;
 	}
@@ -1513,13 +1523,24 @@ out_complete:
 static int dri2_server_generation;
 #endif
 
+static int has_i830_dri(void)
+{
+	return access(DRI_DRIVER_PATH "/i830_dri.so", R_OK) == 0;
+}
+
 static const char *dri_driver_name(intel_screen_private *intel)
 {
 	const char *s = xf86GetOptValString(intel->Options, OPTION_DRI);
 	Bool dummy;
 
-	if (s == NULL || xf86getBoolValue(&dummy, s))
-		return INTEL_INFO(intel)->gen < 040 ? "i915" : "i965";
+	if (s == NULL || xf86getBoolValue(&dummy, s)) {
+		if (INTEL_INFO(intel)->gen < 030)
+			return has_i830_dri() ? "i830" : "i915";
+		else if (INTEL_INFO(intel)->gen < 040)
+			return "i915";
+		else
+			return "i965";
+	}
 
 	return s;
 }
